Package: CrispRVariants
Type: Package
Title: Tools for counting and visualising mutations in a target
        location
Version: 1.40.0
Authors@R: c(person(given = "Helen", family = "Lindsay", role = c("aut", "cre"), email =
          "helen.lindsay@chuv.ch"))
Author: Helen Lindsay [aut, cre]
Description: CrispRVariants provides tools for analysing the results of a
    CRISPR-Cas9 mutagenesis sequencing experiment, or other sequencing experiments
    where variants within a given region are of interest. These tools allow users to
    localize variant allele combinations with respect to any genomic location (e.g.
    the Cas9 cut site), plot allele combinations and calculate mutation rates with
    flexible filtering of unrelated variants.
biocViews: ImmunoOncology, CRISPR, GenomicVariation, VariantDetection,
        GeneticVariability, DataRepresentation, Visualization,
        Sequencing
Depends: R (>= 4.3.0), ggplot2 (>= 2.2.0)
Encoding: UTF-8
License: GPL-2
Imports: AnnotationDbi, BiocParallel, Biostrings, methods,
        GenomeInfoDb, GenomicAlignments, GenomicRanges, grDevices,
        grid, gridExtra, IRanges, reshape2, Rsamtools, S4Vectors (>=
        0.9.38), utils
Suggests: BiocStyle, GenomicFeatures, knitr, rmarkdown, readxl,
        rtracklayer, sangerseqR, testthat, VariantAnnotation
VignetteBuilder: knitr
NeedsCompilation: no
RoxygenNote: 7.2.3
git_url: https://git.bioconductor.org/packages/CrispRVariants
git_branch: RELEASE_3_23
git_last_commit: c855e6f
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
Packaged: 2026-04-28 21:29:19 UTC; biocbuild
Maintainer: Helen Lindsay <helen.lindsay@chuv.ch>
