#' Variant sequences from golden clutch 1 (Burger et al)
#'
#' This dataset is a subset of the crispant data for the golden gene used by Burger et al (submitted).
#'
#' \itemize{
#' \item gol The variants as a CrisprSet object
#' }
#'
#' @docType data
#' @keywords datasets
#' @name gol_clutch1
#' @usage data(gol_clutch1)
#' @format A CrisprSet object countaining 8 samples
#' @aliases gol
#' @return A CrisprSet object named "gol"
NULL